# Integration test for ommers and re-orgs

## Invoking this framework

Run `./test`.

Here, `docker-compose` spins up a CL node and a POA geth node. The geth node always begins mining from the genesis block; it has no persisted state. The CL node records data in `tmp/clroot`. First, we create an EthLog job and then a contract that emits an event when it is created. See test_helpers `create_job` and `create_contract`. Then, we disconnect CL from the blockchain before the minimum # of block confirmations (10) is reached. Next, we restart the blockchain and start mining from genesis again. Eventually, the new chain becomes longer than the old chain and the CL node should abandon the job run. We test for this by looking for "presumably has been uncled" in the logs.

Currently, the contract's bytecode is hard-coded in `fixtures/create_contract.json`. In the future, this test should be extended to actually compile a .sol file so that the contract can easily be changed in the future.

## Geth Configuration

The geth node relies on two files for configuration:

- `geth_node/root/genesis.json`
- `geth_note/root/geth-config.toml`

The first is generated by running `puppeth` (requires full [geth toolkit](puppeth)) and filling in the answers appropriately. The import section, shown below, tells geth to start a POA node and to mine a block every 1 second.

```json
{
  "clique": {
    "period": 1,
    "epoch": 30000
  }
}
```

We also seed the chainlink account, `0xb90c7E3F7815F59EAD74e7543eB6D9E8538455D6`, with some initial ETH like so:

```json
{
  "alloc": {
    "b90c7E3F7815F59EAD74e7543eB6D9E8538455D6": {
      "balance": "0x10000000000000000000000000"
    }
  }
}
```

The boilerplate `geth-config.toml` file is generated by running `geth dumpconfig > <filename>`. See the file for explanations of more configuration options.

## Logs

The chainlink logs are available from each run in `logs/chain_1.log` and `logs/chain_2.log`. The logs should contain something similar to the following:

#### Chain 1 Log

```
[DEBUG] Received new head #6 (0x6)                         services/head_tracker.go:208     blockHash=0x2e9fb117886c3ed154b5e4a37cf3f6ff3ad79afa6ab99da4484f13d9c9229c6c blockHeight=6
[DEBUG] Log for ethlog initiator for job 0549b37324664fb59060b59e7241d89b services/subscription.go:109     blockNumber=6 job=0549b37324664fb59060b59e7241d89b logIndex=0 txHash=0x01aea35b8e9f3d29d6216202aebea63fc4d823b2ac8ace2d9a757fe9a8491634
[DEBUG] Received log from block #6 for address [all] for job 0549b37324664fb59060b59e7241d89b models/log_events.go:189         initiator={"id":1,"jobSpecId":"0549b37324664fb59060b59e7241d89b","type":"ethlog","CreatedAt":"2019-10-16T21:38:26.326364195Z","params":{"time":null,"address":"0x0000000000000000000000000000000000000000"}} job=0549b37324664fb59060b59e7241d89b log=6 topic0=0xa9f7ac292bec89f3379964cf3da3a5dec83186cd18c08abe2c0603a06fb19c29
[DEBUG] New run triggered by ethlog                        services/runs.go:45              creation_height=6 job=0549b37324664fb59060b59e7241d89b
[DEBUG] Run cannot continue because it lacks sufficient confirmations services/runs.go:359             required_height=10 run=0fa99142dcdd42ada8bbaffabb6defaf
[DEBUG] Pausing run originally initiated by ethlog         services/runs.go:448             creation_height=6 job=0549b37324664fb59060b59e7241d89b observed_height=6 run=0fa99142dcdd42ada8bbaffabb6defaf status=pending_confirmations
[DEBUG] Received new head #7 (0x7)                         services/head_tracker.go:208     blockHash=0xaa1e33b36eb24acefe0bc475175b9ab497545af8581a097cc655cebfc040a074 blockHeight=7
[DEBUG] New head resuming run                              services/runs.go:181             creation_height=6 job=0549b37324664fb59060b59e7241d89b observed_height=6 run=0fa99142dcdd42ada8bbaffabb6defaf status=pending_confirmations
[DEBUG] Run cannot continue because it lacks sufficient confirmations services/runs.go:359             required_height=10 run=0fa99142dcdd42ada8bbaffabb6defaf
[DEBUG] Pausing run originally initiated by ethlog         services/runs.go:448             creation_height=6 job=0549b37324664fb59060b59e7241d89b observed_height=7 run=0fa99142dcdd42ada8bbaffabb6defaf status=pending_confirmations
```

#### Chain 2 Log

```
[DEBUG] Received new head #7 (0x7)                         services/head_tracker.go:208     blockHash=0xe8b2925307b67141a76c89134f6c6fefc5b6c8517cb6f75c9d13e6efcae979b6 blockHeight=7
[WARN]  Cannot save new head confirmation 7 because it's equal to or less than current head 8 with hash 0xe8b2925307b67141a76c89134f6c6fefc5b6c8517cb6f75c9d13e6efcae979b6 services/head_tracker.go:216
[DEBUG] Received new head #8 (0x8)                         services/head_tracker.go:208     blockHash=0x7afc104797dffd0fe7fffe137abaee31c072db8ea4150493cf4c54527876da36 blockHeight=8
[WARN]  Cannot save new head confirmation 8 because it's equal to or less than current head 8 with hash 0x7afc104797dffd0fe7fffe137abaee31c072db8ea4150493cf4c54527876da36 services/head_tracker.go:216
[DEBUG] Received new head #9 (0x9)                         services/head_tracker.go:208     blockHash=0xf825d1e2bda0dbab46f46b1624b75cce7d67b81f8f409a86dae356b37fd70182 blockHeight=9
[DEBUG] New head resuming run                              services/runs.go:181             creation_height=6 job=0549b37324664fb59060b59e7241d89b observed_height=8 run=0fa99142dcdd42ada8bbaffabb6defaf status=pending_confirmations
[DEBUG] Run cannot continue because it lacks sufficient confirmations services/runs.go:359             required_height=10 run=0fa99142dcdd42ada8bbaffabb6defaf
[DEBUG] Pausing run originally initiated by ethlog         services/runs.go:448             creation_height=6 job=0549b37324664fb59060b59e7241d89b observed_height=9 run=0fa99142dcdd42ada8bbaffabb6defaf status=pending_confirmations

...

[DEBUG] Received new head #15 (0xf)                        services/head_tracker.go:208     blockHash=0x721cb65082acf9791ebcfa12ff802b30d63f4e1a3a9942ab4c0c8a01a90eb7af blockHeight=15
[DEBUG] New head resuming run                              services/runs.go:181             creation_height=6 job=0549b37324664fb59060b59e7241d89b observed_height=14 run=0fa99142dcdd42ada8bbaffabb6defaf status=pending_confirmations
[DEBUG] Pausing run originally initiated by ethlog         services/runs.go:448             creation_height=6 job=0549b37324664fb59060b59e7241d89b job_error=TxHash 0x01aea35b8e9f3d29d6216202aebea63fc4d823b2ac8ace2d9a757fe9a8491634 initiating run 0fa99142dcdd42ada8bbaffabb6defaf not on main chain; presumably has been uncled observed_height=15 run=0fa99142dcdd42ada8bbaffabb6defaf status=errored
[DEBUG] Received new head #16 (0x10)                       services/head_tracker.go:208     blockHash=0x26e5f3a0df34b4d029ac0fa1fb4c8f0ca617e6111277a1a209d86a21b48e2434 blockHeight=16
[DEBUG] Received new head #17 (0x11)                       services/head_tracker.go:208     blockHash=0x5e94cd532c9d7b80ba84d59e7ef7ca46ae3e8b1f9b0f3982fed6b7e8f548926a blockHeight=17
```

[puppeth]: https://github.com/ethereum/go-ethereum#executables
